@@ -14,6 +14,5 @@ SNAPSHOT_TIMEOUT = 20
 SERVICE_EVENT = "event"
 SERVICE_UPDATE = "update"
 
-SENSOR_DEVICE_CLASS = "class"
-SENSOR_EVENT_CODE = "code"
-SENSOR_NAME = "name"
+RESOLUTION_LIST = {"high": 0, "low": 1}
+RESOLUTION_TO_STREAM = {0: "Main", 1: "Extra"}
