@@ -20,10 +20,10 @@ from fritzconnection.lib.fritzhosts import FritzHosts
 from fritzconnection.lib.fritzstatus import FritzStatus
 from fritzconnection.lib.fritzwlan import DEFAULT_PASSWORD_LENGTH, FritzGuestWLAN
 
-from homeassistant.components.device_tracker import DOMAIN as DEVICE_TRACKER_DOMAIN
-from homeassistant.components.device_tracker.const import (
+from homeassistant.components.device_tracker import (
     CONF_CONSIDER_HOME,
     DEFAULT_CONSIDER_HOME,
+    DOMAIN as DEVICE_TRACKER_DOMAIN,
 )
 from homeassistant.components.switch import DOMAIN as DEVICE_SWITCH_DOMAIN
 from homeassistant.config_entries import ConfigEntry
@@ -91,6 +91,11 @@ def _cleanup_entity_filter(device: er.RegistryEntry) -> bool:
     )
 
 
+def _ha_is_stopping(activity: str) -> None:
+    """Inform that HA is stopping."""
+    _LOGGER.info("Cannot execute %s: HomeAssistant is shutting down", activity)
+
+
 class ClassSetupMissing(Exception):
     """Raised when a Class func is called before setup."""
 
@@ -169,6 +174,7 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
         self._current_firmware: str | None = None
         self._latest_firmware: str | None = None
         self._update_available: bool = False
+        self._release_url: str | None = None
 
     async def async_setup(
         self, options: MappingProxyType[str, Any] | None = None
@@ -201,36 +207,36 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
         self.fritz_hosts = FritzHosts(fc=self.connection)
         self.fritz_guest_wifi = FritzGuestWLAN(fc=self.connection)
         self.fritz_status = FritzStatus(fc=self.connection)
-        info = self.connection.call_action("DeviceInfo:1", "GetInfo")
+        info = self.fritz_status.get_device_info()
 
         _LOGGER.debug(
             "gathered device info of %s %s",
             self.host,
             {
-                **info,
+                **vars(info),
                 "NewDeviceLog": "***omitted***",
                 "NewSerialNumber": "***omitted***",
             },
         )
 
         if not self._unique_id:
-            self._unique_id = info["NewSerialNumber"]
+            self._unique_id = info.serial_number
 
-        self._model = info.get("NewModelName")
-        self._current_firmware = info.get("NewSoftwareVersion")
+        self._model = info.model_name
+        self._current_firmware = info.software_version
 
-        self._update_available, self._latest_firmware = self._update_device_info()
-        if "Layer3Forwarding1" in self.connection.services:
-            if connection_type := self.connection.call_action(
-                "Layer3Forwarding1", "GetDefaultConnectionService"
-            ).get("NewDefaultConnectionService"):
-                # Return NewDefaultConnectionService sample: "1.WANPPPConnection.1"
-                self.device_conn_type = connection_type[2:][:-2]
-                self.device_is_router = self.connection.call_action(
-                    self.device_conn_type, "GetInfo"
-                ).get("NewEnable")
+        (
+            self._update_available,
+            self._latest_firmware,
+            self._release_url,
+        ) = self._update_device_info()
+
+        if self.fritz_status.has_wan_support:
+            self.device_conn_type = (
+                self.fritz_status.get_default_connection_service().connection_service
+            )
+            self.device_is_router = self.fritz_status.has_wan_enabled
 
-    @callback
     async def _async_update_data(self) -> None:
         """Update FritzboxTools data."""
         try:
@@ -269,6 +275,11 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
         """Return if new SW version is available."""
         return self._update_available
 
+    @property
+    def release_url(self) -> str | None:
+        """Return the info URL for latest firmware."""
+        return self._release_url
+
     @property
     def mac(self) -> str:
         """Return device Mac address."""
@@ -300,12 +311,12 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
                 raise HomeAssistantError("Error refreshing hosts info") from ex
         return []
 
-    def _update_device_info(self) -> tuple[bool, str | None]:
+    def _update_device_info(self) -> tuple[bool, str | None, str | None]:
         """Retrieve latest device information from the FRITZ!Box."""
-        version = self.connection.call_action("UserInterface1", "GetInfo").get(
-            "NewX_AVM-DE_Version"
-        )
-        return bool(version), version
+        info = self.connection.call_action("UserInterface1", "GetInfo")
+        version = info.get("NewX_AVM-DE_Version")
+        release_url = info.get("NewX_AVM-DE_InfoURL")
+        return bool(version), version, release_url
 
     def _get_wan_access(self, ip_address: str) -> bool | None:
         """Get WAN access rule for given IP address."""
@@ -351,8 +362,16 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
     def scan_devices(self, now: datetime | None = None) -> None:
         """Scan for new devices and return a list of found device ids."""
 
+        if self.hass.is_stopping:
+            _ha_is_stopping("scan devices")
+            return
+
         _LOGGER.debug("Checking host info for FRITZ!Box device %s", self.host)
-        self._update_available, self._latest_firmware = self._update_device_info()
+        (
+            self._update_available,
+            self._latest_firmware,
+            self._release_url,
+        ) = self._update_device_info()
 
         _LOGGER.debug("Checking devices for FRITZ!Box device %s", self.host)
         _default_consider_home = DEFAULT_CONSIDER_HOME.total_seconds()
@@ -379,11 +398,7 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
                 wan_access=None,
             )
 
-        if (
-            "Hosts1" not in self.connection.services
-            or "X_AVM-DE_GetMeshListPath"
-            not in self.connection.services["Hosts1"].actions
-        ) or (
+        if not self.fritz_status.device_has_mesh_support or (
             self._options
             and self._options.get(CONF_OLD_DISCOVERY, DEFAULT_CONF_OLD_DISCOVERY)
         ):
@@ -392,6 +407,8 @@ class FritzBoxTools(update_coordinator.DataUpdateCoordinator):
             )
             self.mesh_role = MeshRoles.NONE
             for mac, info in hosts.items():
+                if info.ip_address:
+                    info.wan_access = self._get_wan_access(info.ip_address)
                 if self.manage_device_info(info, mac, consider_home):
                     new_device = True
             self.send_signal_device_update(new_device)
@@ -601,6 +618,10 @@ class AvmWrapper(FritzBoxTools):
     ) -> dict:
         """Return service details."""
 
+        if self.hass.is_stopping:
+            _ha_is_stopping(f"{service_name}/{action_name}")
+            return {}
+
         if f"{service_name}{service_suffix}" not in self.connection.services:
             return {}
 
@@ -642,6 +663,34 @@ class AvmWrapper(FritzBoxTools):
             partial(self.get_wan_link_properties)
         )
 
+    async def async_ipv6_active(self) -> bool:
+        """Check ip an ipv6 is active on the WAn interface."""
+
+        def wrap_external_ipv6() -> str:
+            return str(self.fritz_status.external_ipv6)
+
+        if not self.device_is_router:
+            return False
+
+        return bool(await self.hass.async_add_executor_job(wrap_external_ipv6))
+
+    async def async_get_connection_info(self) -> ConnectionInfo:
+        """Return ConnectionInfo data."""
+
+        link_properties = await self.async_get_wan_link_properties()
+        connection_info = ConnectionInfo(
+            connection=link_properties.get("NewWANAccessType", "").lower(),
+            mesh_role=self.mesh_role,
+            wan_enabled=self.device_is_router,
+            ipv6_active=await self.async_ipv6_active(),
+        )
+        _LOGGER.debug(
+            "ConnectionInfo for FritzBox %s: %s",
+            self.host,
+            connection_info,
+        )
+        return connection_info
+
     async def async_get_port_mapping(self, con_type: str, index: int) -> dict[str, Any]:
         """Call GetGenericPortMappingEntry action."""
 
@@ -799,7 +848,7 @@ class FritzData:
     profile_switches: dict = field(default_factory=dict)
 
 
-class FritzDeviceBase(update_coordinator.CoordinatorEntity):
+class FritzDeviceBase(update_coordinator.CoordinatorEntity[AvmWrapper]):
     """Entity base class for a device connected to a FRITZ!Box device."""
 
     def __init__(self, avm_wrapper: AvmWrapper, device: FritzDevice) -> None:
@@ -833,11 +882,6 @@ class FritzDeviceBase(update_coordinator.CoordinatorEntity):
             return self._avm_wrapper.devices[self._mac].hostname
         return None
 
-    @property
-    def should_poll(self) -> bool:
-        """No polling needed."""
-        return False
-
     async def async_process_update(self) -> None:
         """Update device."""
         raise NotImplementedError()
@@ -970,3 +1014,13 @@ class FritzBoxBaseEntity:
             name=self._device_name,
             sw_version=self._avm_wrapper.current_firmware,
         )
+
+
+@dataclass
+class ConnectionInfo:
+    """Fritz sensor connection information class."""
+
+    connection: str
+    mesh_role: MeshRoles
+    wan_enabled: bool
+    ipv6_active: bool
