@@ -7,7 +7,10 @@ from uuid import UUID
 
 from pyhap.accessory import Accessory, Bridge
 from pyhap.accessory_driver import AccessoryDriver
+from pyhap.characteristic import Characteristic
 from pyhap.const import CATEGORY_OTHER
+from pyhap.iid_manager import IIDManager
+from pyhap.service import Service
 from pyhap.util import callback as pyhap_callback
 
 from homeassistant.components.cover import CoverDeviceClass, CoverEntityFeature
@@ -83,6 +86,7 @@ from .const import (
     TYPE_SWITCH,
     TYPE_VALVE,
 )
+from .iidmanager import AccessoryIIDStorage
 from .util import (
     accessory_friendly_name,
     async_dismiss_setup_message,
@@ -196,6 +200,10 @@ def get_accessory(  # noqa: C901
             or SensorDeviceClass.PM25 in state.entity_id
         ):
             a_type = "PM25Sensor"
+        elif device_class == SensorDeviceClass.NITROGEN_DIOXIDE:
+            a_type = "NitrogenDioxideSensor"
+        elif device_class == SensorDeviceClass.VOLATILE_ORGANIC_COMPOUNDS:
+            a_type = "VolatileOrganicCompoundsSensor"
         elif (
             device_class == SensorDeviceClass.GAS
             or SensorDeviceClass.GAS in state.entity_id
@@ -266,6 +274,7 @@ class HomeAccessory(Accessory):  # type: ignore[misc]
             driver=driver,
             display_name=cleanup_name_for_homekit(name),
             aid=aid,
+            iid_manager=HomeIIDManager(driver.iid_storage),
             *args,
             **kwargs,
         )
@@ -316,8 +325,8 @@ class HomeAccessory(Accessory):  # type: ignore[misc]
             serv_info.configure_char(
                 CHAR_HARDWARE_REVISION, value=hw_version[:MAX_VERSION_LENGTH]
             )
-            self.iid_manager.assign(char)
             char.broker = self
+            self.iid_manager.assign(char)
 
         self.category = category
         self.entity_id = entity_id
@@ -349,12 +358,12 @@ class HomeAccessory(Accessory):  # type: ignore[misc]
             if state is not None:
                 battery_found = state.state
             else:
-                self.linked_battery_sensor = None
                 _LOGGER.warning(
                     "%s: Battery sensor state missing: %s",
                     self.entity_id,
                     self.linked_battery_sensor,
                 )
+                self.linked_battery_sensor = None
 
         if not battery_found:
             return
@@ -565,7 +574,7 @@ class HomeBridge(Bridge):  # type: ignore[misc]
 
     def __init__(self, hass: HomeAssistant, driver: HomeDriver, name: str) -> None:
         """Initialize a Bridge object."""
-        super().__init__(driver, name)
+        super().__init__(driver, name, iid_manager=HomeIIDManager(driver.iid_storage))
         self.set_info_service(
             firmware_revision=format_version(__version__),
             manufacturer=MANUFACTURER,
@@ -598,6 +607,7 @@ class HomeDriver(AccessoryDriver):  # type: ignore[misc]
         entry_id: str,
         bridge_name: str,
         entry_title: str,
+        iid_storage: AccessoryIIDStorage,
         **kwargs: Any,
     ) -> None:
         """Initialize a AccessoryDriver object."""
@@ -606,6 +616,7 @@ class HomeDriver(AccessoryDriver):  # type: ignore[misc]
         self._entry_id = entry_id
         self._bridge_name = bridge_name
         self._entry_title = entry_title
+        self.iid_storage = iid_storage
 
     @pyhap_callback  # type: ignore[misc]
     def pair(
@@ -632,3 +643,30 @@ class HomeDriver(AccessoryDriver):  # type: ignore[misc]
             self.state.pincode,
             self.accessory.xhm_uri(),
         )
+
+
+class HomeIIDManager(IIDManager):  # type: ignore[misc]
+    """IID Manager that remembers IIDs between restarts."""
+
+    def __init__(self, iid_storage: AccessoryIIDStorage) -> None:
+        """Initialize a IIDManager object."""
+        super().__init__()
+        self._iid_storage = iid_storage
+
+    def get_iid_for_obj(self, obj: Characteristic | Service) -> int:
+        """Get IID for object."""
+        aid = obj.broker.aid
+        if isinstance(obj, Characteristic):
+            service: Service = obj.service
+            iid = self._iid_storage.get_or_allocate_iid(
+                aid, service.type_id, service.unique_id, obj.type_id, obj.unique_id
+            )
+        else:
+            iid = self._iid_storage.get_or_allocate_iid(
+                aid, obj.type_id, obj.unique_id, None, None
+            )
+        if iid in self.objs:
+            raise RuntimeError(
+                f"Cannot assign IID {iid} to {obj} as it is already in use by: {self.objs[iid]}"
+            )
+        return iid
