@@ -1,7 +1,9 @@
 """Support for MQTT locks."""
 from __future__ import annotations
 
+from collections.abc import Callable
 import functools
+from typing import Any
 
 import voluptuous as vol
 
@@ -28,11 +30,10 @@ from .mixins import (
     MQTT_ENTITY_COMMON_SCHEMA,
     MqttEntity,
     async_setup_entry_helper,
-    async_setup_platform_discovery,
-    async_setup_platform_helper,
     warn_for_legacy_schema,
 )
-from .models import MqttValueTemplate
+from .models import MqttValueTemplate, ReceiveMessage, ReceivePayloadType
+from .util import get_mqtt_data
 
 CONF_PAYLOAD_LOCK = "payload_lock"
 CONF_PAYLOAD_UNLOCK = "payload_unlock"
@@ -69,43 +70,21 @@ PLATFORM_SCHEMA_MODERN = MQTT_RW_SCHEMA.extend(
     }
 ).extend(MQTT_ENTITY_COMMON_SCHEMA.schema)
 
-# Configuring MQTT Locks under the lock platform key is deprecated in HA Core 2022.6
+# Configuring MQTT Locks under the lock platform key was deprecated in HA Core 2022.6
+# Setup for the legacy YAML format was removed in HA Core 2022.12
 PLATFORM_SCHEMA = vol.All(
-    cv.PLATFORM_SCHEMA.extend(PLATFORM_SCHEMA_MODERN.schema),
     warn_for_legacy_schema(lock.DOMAIN),
 )
 
 DISCOVERY_SCHEMA = PLATFORM_SCHEMA_MODERN.extend({}, extra=vol.REMOVE_EXTRA)
 
 
-async def async_setup_platform(
-    hass: HomeAssistant,
-    config: ConfigType,
-    async_add_entities: AddEntitiesCallback,
-    discovery_info: DiscoveryInfoType | None = None,
-) -> None:
-    """Set up MQTT locks configured under the lock platform key (deprecated)."""
-    # Deprecated in HA Core 2022.6
-    await async_setup_platform_helper(
-        hass,
-        lock.DOMAIN,
-        discovery_info or config,
-        async_add_entities,
-        _async_setup_entity,
-    )
-
-
 async def async_setup_entry(
     hass: HomeAssistant,
     config_entry: ConfigEntry,
     async_add_entities: AddEntitiesCallback,
 ) -> None:
     """Set up MQTT lock through configuration.yaml and dynamically through MQTT discovery."""
-    # load and initialize platform config from configuration.yaml
-    config_entry.async_on_unload(
-        await async_setup_platform_discovery(hass, lock.DOMAIN)
-    )
-    # setup for discovery
     setup = functools.partial(
         _async_setup_entity, hass, async_add_entities, config_entry=config_entry
     )
@@ -113,8 +92,12 @@ async def async_setup_entry(
 
 
 async def _async_setup_entity(
-    hass, async_add_entities, config, config_entry=None, discovery_data=None
-):
+    hass: HomeAssistant,
+    async_add_entities: AddEntitiesCallback,
+    config: ConfigType,
+    config_entry: ConfigEntry,
+    discovery_data: DiscoveryInfoType | None = None,
+) -> None:
     """Set up the MQTT Lock platform."""
     async_add_entities([MqttLock(hass, config, config_entry, discovery_data)])
 
@@ -125,41 +108,52 @@ class MqttLock(MqttEntity, LockEntity):
     _entity_id_format = lock.ENTITY_ID_FORMAT
     _attributes_extra_blocked = MQTT_LOCK_ATTRIBUTES_BLOCKED
 
-    def __init__(self, hass, config, config_entry, discovery_data):
-        """Initialize the lock."""
-        self._state = False
-        self._optimistic = False
+    _optimistic: bool
+    _value_template: Callable[[ReceivePayloadType], ReceivePayloadType]
 
+    def __init__(
+        self,
+        hass: HomeAssistant,
+        config: ConfigType,
+        config_entry: ConfigEntry,
+        discovery_data: DiscoveryInfoType | None,
+    ) -> None:
+        """Initialize the lock."""
+        self._attr_is_locked = False
         MqttEntity.__init__(self, hass, config, config_entry, discovery_data)
 
     @staticmethod
-    def config_schema():
+    def config_schema() -> vol.Schema:
         """Return the config schema."""
         return DISCOVERY_SCHEMA
 
-    def _setup_from_config(self, config):
+    def _setup_from_config(self, config: ConfigType) -> None:
         """(Re)Setup the entity."""
         self._optimistic = config[CONF_OPTIMISTIC]
 
         self._value_template = MqttValueTemplate(
-            self._config.get(CONF_VALUE_TEMPLATE),
+            config.get(CONF_VALUE_TEMPLATE),
             entity=self,
         ).async_render_with_possible_json_value
 
-    def _prepare_subscribe_topics(self):
+        self._attr_supported_features = LockEntityFeature(0)
+        if CONF_PAYLOAD_OPEN in config:
+            self._attr_supported_features |= LockEntityFeature.OPEN
+
+    def _prepare_subscribe_topics(self) -> None:
         """(Re)Subscribe to topics."""
 
         @callback
         @log_messages(self.hass, self.entity_id)
-        def message_received(msg):
+        def message_received(msg: ReceiveMessage) -> None:
             """Handle new MQTT messages."""
             payload = self._value_template(msg.payload)
             if payload == self._config[CONF_STATE_LOCKED]:
-                self._state = True
+                self._attr_is_locked = True
             elif payload == self._config[CONF_STATE_UNLOCKED]:
-                self._state = False
+                self._attr_is_locked = False
 
-            self.async_write_ha_state()
+            get_mqtt_data(self.hass).state_write_requests.write_state_request(self)
 
         if self._config.get(CONF_STATE_TOPIC) is None:
             # Force into optimistic mode.
@@ -178,26 +172,16 @@ class MqttLock(MqttEntity, LockEntity):
                 },
             )
 
-    async def _subscribe_topics(self):
+    async def _subscribe_topics(self) -> None:
         """(Re)Subscribe to topics."""
         await subscription.async_subscribe_topics(self.hass, self._sub_state)
 
     @property
-    def is_locked(self):
-        """Return true if lock is locked."""
-        return self._state
-
-    @property
-    def assumed_state(self):
+    def assumed_state(self) -> bool:
         """Return true if we do optimistic updates."""
         return self._optimistic
 
-    @property
-    def supported_features(self):
-        """Flag supported features."""
-        return LockEntityFeature.OPEN if CONF_PAYLOAD_OPEN in self._config else 0
-
-    async def async_lock(self, **kwargs):
+    async def async_lock(self, **kwargs: Any) -> None:
         """Lock the device.
 
         This method is a coroutine.
@@ -211,10 +195,10 @@ class MqttLock(MqttEntity, LockEntity):
         )
         if self._optimistic:
             # Optimistically assume that the lock has changed state.
-            self._state = True
+            self._attr_is_locked = True
             self.async_write_ha_state()
 
-    async def async_unlock(self, **kwargs):
+    async def async_unlock(self, **kwargs: Any) -> None:
         """Unlock the device.
 
         This method is a coroutine.
@@ -228,10 +212,10 @@ class MqttLock(MqttEntity, LockEntity):
         )
         if self._optimistic:
             # Optimistically assume that the lock has changed state.
-            self._state = False
+            self._attr_is_locked = False
             self.async_write_ha_state()
 
-    async def async_open(self, **kwargs):
+    async def async_open(self, **kwargs: Any) -> None:
         """Open the door latch.
 
         This method is a coroutine.
@@ -245,5 +229,5 @@ class MqttLock(MqttEntity, LockEntity):
         )
         if self._optimistic:
             # Optimistically assume that the lock unlocks when opened.
-            self._state = False
+            self._attr_is_locked = False
             self.async_write_ha_state()
