@@ -1,4 +1,6 @@
 """Support for OpenCV classification on images."""
+from __future__ import annotations
+
 from datetime import timedelta
 import logging
 
@@ -7,14 +9,14 @@ import requests
 import voluptuous as vol
 
 from homeassistant.components.image_processing import (
-    CONF_ENTITY_ID,
-    CONF_NAME,
-    CONF_SOURCE,
     PLATFORM_SCHEMA,
     ImageProcessingEntity,
 )
-from homeassistant.core import split_entity_id
+from homeassistant.const import CONF_ENTITY_ID, CONF_NAME, CONF_SOURCE
+from homeassistant.core import HomeAssistant, split_entity_id
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
 try:
     # Verify that the OpenCV python package is pre-installed
@@ -85,14 +87,19 @@ def _create_processor_from_config(hass, camera_entity, config):
 def _get_default_classifier(dest_path):
     """Download the default OpenCV classifier."""
     _LOGGER.info("Downloading default classifier")
-    req = requests.get(CASCADE_URL, stream=True)
+    req = requests.get(CASCADE_URL, stream=True, timeout=10)
     with open(dest_path, "wb") as fil:
         for chunk in req.iter_content(chunk_size=1024):
             if chunk:  # filter out keep-alive new chunks
                 fil.write(chunk)
 
 
-def setup_platform(hass, config, add_entities, discovery_info=None):
+def setup_platform(
+    hass: HomeAssistant,
+    config: ConfigType,
+    add_entities: AddEntitiesCallback,
+    discovery_info: DiscoveryInfoType | None = None,
+) -> None:
     """Set up the OpenCV image processing platform."""
     if not CV2_IMPORTED:
         _LOGGER.error(
