@@ -1,12 +1,14 @@
 """Helpers to install PyPi packages."""
+from __future__ import annotations
+
 import asyncio
+from functools import cache
 from importlib.metadata import PackageNotFoundError, version
 import logging
 import os
 from pathlib import Path
 from subprocess import PIPE, Popen
 import sys
-from typing import Optional
 from urllib.parse import urlparse
 
 import pkg_resources
@@ -22,6 +24,7 @@ def is_virtual_env() -> bool:
     )
 
 
+@cache
 def is_docker_env() -> bool:
     """Return True if we run in a docker env."""
     return Path("/.dockerenv").exists()
@@ -34,6 +37,9 @@ def is_installed(package: str) -> bool:
     Returns False when the package is not installed or doesn't meet req.
     """
     try:
+        pkg_resources.get_distribution(package)
+        return True
+    except (pkg_resources.ResolutionError, pkg_resources.ExtractionError):
         req = pkg_resources.Requirement.parse(package)
     except ValueError:
         # This is a zip file. We no longer use this in Home Assistant,
@@ -41,7 +47,14 @@ def is_installed(package: str) -> bool:
         req = pkg_resources.Requirement.parse(urlparse(package).fragment)
 
     try:
-        return version(req.project_name) in req
+        installed_version = version(req.project_name)
+        # This will happen when an install failed or
+        # was aborted while in progress see
+        # https://github.com/home-assistant/core/issues/47699
+        if installed_version is None:
+            _LOGGER.error("Installed version for %s resolved to None", req.project_name)  # type: ignore[unreachable]
+            return False
+        return installed_version in req
     except PackageNotFoundError:
         return False
 
@@ -49,10 +62,11 @@ def is_installed(package: str) -> bool:
 def install_package(
     package: str,
     upgrade: bool = True,
-    target: Optional[str] = None,
-    constraints: Optional[str] = None,
-    find_links: Optional[str] = None,
-    no_cache_dir: Optional[bool] = False,
+    target: str | None = None,
+    constraints: str | None = None,
+    find_links: str | None = None,
+    timeout: int | None = None,
+    no_cache_dir: bool | None = False,
 ) -> bool:
     """Install a package on PyPi. Accepts pip compatible package strings.
 
@@ -62,6 +76,8 @@ def install_package(
     _LOGGER.info("Attempting install of %s", package)
     env = os.environ.copy()
     args = [sys.executable, "-m", "pip", "install", "--quiet", package]
+    if timeout:
+        args += ["--timeout", str(timeout)]
     if no_cache_dir:
         args.append("--no-cache-dir")
     if upgrade:
@@ -75,19 +91,16 @@ def install_package(
         # This only works if not running in venv
         args += ["--user"]
         env["PYTHONUSERBASE"] = os.path.abspath(target)
-        if sys.platform != "win32":
-            # Workaround for incompatible prefix setting
-            # See http://stackoverflow.com/a/4495175
-            args += ["--prefix="]
-    process = Popen(args, stdin=PIPE, stdout=PIPE, stderr=PIPE, env=env)
-    _, stderr = process.communicate()
-    if process.returncode != 0:
-        _LOGGER.error(
-            "Unable to install package %s: %s",
-            package,
-            stderr.decode("utf-8").lstrip().strip(),
-        )
-        return False
+    _LOGGER.debug("Running pip command: args=%s", args)
+    with Popen(args, stdin=PIPE, stdout=PIPE, stderr=PIPE, env=env) as process:
+        _, stderr = process.communicate()
+        if process.returncode != 0:
+            _LOGGER.error(
+                "Unable to install package %s: %s",
+                package,
+                stderr.decode("utf-8").lstrip().strip(),
+            )
+            return False
 
     return True
 
